import { Router, Request, Response } from 'express'
import fs from 'fs'
import { httpLogger } from '../utilities/logger'
import { getPublicPath } from '../utilities/storage'
import TileRepository from '../repositories/tileRepository'
import ZoneRepository from '../repositories/zoneRepository'
import SpriteRepository from '../repositories/spriteRepository'
import { AssetData } from '../utilities/types'

const router = Router()

// Get all tiles
router.get('/assets/list_tiles', async (req: Request, res: Response) => {
  let assets: AssetData[] = []
  const tiles = await TileRepository.getAll()
  for (const tile of tiles) {
    assets.push({
      key: tile.id,
      data: '/assets/tiles/' + tile.id + '.png',
      group: 'tiles',
      updatedAt: tile.updatedAt
    } as AssetData)
  }
  res.json(assets)
})

// Get tiles by zone
router.get('/assets/list_tiles/:zoneId', async (req: Request, res: Response) => {
  const zoneId = req.params.zoneId

  if (!zoneId || parseInt(zoneId) === 0) {
    return res.status(400).json({ message: 'Invalid zone ID' })
  }

  const zone = await ZoneRepository.getById(parseInt(zoneId))
  if (!zone) {
    return res.status(404).json({ message: 'Zone not found' })
  }

  let assets: AssetData[] = []
  const tiles = await TileRepository.getByZoneId(parseInt(zoneId))
  for (const tile of tiles) {
    assets.push({
      key: tile.id,
      data: '/assets/tiles/' + tile.id + '.png',
      group: 'tiles',
      updatedAt: tile.updatedAt
    } as AssetData)
  }

  res.json(assets)
})

// Get sprite actions
router.get('/assets/list_sprite_actions/:spriteId', async (req: Request, res: Response) => {
  const spriteId = req.params.spriteId

  if (!spriteId || parseInt(spriteId) === 0) {
    return res.status(400).json({ message: 'Invalid sprite ID' })
  }

  const sprite = await SpriteRepository.getById(spriteId)
  if (!sprite) {
    return res.status(404).json({ message: 'Sprite not found' })
  }

  let assets: AssetData[] = []
  sprite.spriteActions.forEach((spriteAction) => {
    assets.push({
      key: sprite.id + '-' + spriteAction.action,
      data: '/assets/sprites/' + sprite.id + '/' + spriteAction.action + '.png',
      group: spriteAction.isAnimated ? 'sprite_animations' : 'sprites',
      updatedAt: sprite.updatedAt,
      originX: Number(spriteAction.originX.toString()),
      originY: Number(spriteAction.originY.toString()),
      isAnimated: spriteAction.isAnimated,
      frameCount: JSON.parse(JSON.stringify(spriteAction.sprites)).length,
      frameWidth: spriteAction.frameWidth,
      frameHeight: spriteAction.frameHeight,
      frameRate: spriteAction.frameRate
    })
  })

  res.json(assets)
})

// Download asset file
router.get('/assets/:type/:spriteId?/:file', (req: Request, res: Response) => {
  const assetType = req.params.type
  const spriteId = req.params.spriteId
  const fileName = req.params.file

  let assetPath
  if (assetType === 'sprites' && spriteId) {
    assetPath = getPublicPath(assetType, spriteId, fileName)
  } else {
    assetPath = getPublicPath(assetType, fileName)
  }

  if (!fs.existsSync(assetPath)) {
    httpLogger.error(`File not found: ${assetPath}`)
    return res.status(404).send('Asset not found')
  }

  res.sendFile(assetPath, (err) => {
    if (err) {
      httpLogger.error('Error sending file:', err)
      res.status(500).send('Error downloading the asset')
    }
  })
})

export default router
